#include "camera.h"

Camera::Camera() : cameraTheta(0), cameraPhi(0),
	cameraPosition(0, 0, 0),
	cameraForward(0, 1, 0),
	captureInput(false),
	oldCursorX(0), oldCursorY(0), updateVectors(true)
{
}

Camera::~Camera()
{
	release();
}

void Camera::LookAt(Vector3 Eye, Vector3 At, Vector3 Up)
{
	gluLookAt(Eye.m_X, Eye.m_Y, Eye.m_Z, At.m_X, At.m_Y, At.m_Z, Up.m_X, Up.m_Y, Up.m_Z);
}

bool Camera::keyDown(BYTE key)
{
	return (GetAsyncKeyState(key) < 0);
}

void Camera::update()
{
	if(!captureInput) return;

	if(!UpdateVectors())	// No point keeping the mouse captured if we are in 1st person.
	{
		release();
		return;
	}

	// FPS style keys.

	if(keyDown(VK_UP))		cameraPosition += cameraForward;								//
	if(keyDown(VK_LEFT))	cameraPosition -= cameraForward*Vector3(0.0, 0.0, 1.0f);		//
	if(keyDown(VK_DOWN))	cameraPosition -= cameraForward;								//
	if(keyDown(VK_RIGHT))	cameraPosition += cameraForward*Vector3(0.0, 0.0, 1.0f);		//

	//go up and down
	if(keyDown(0x51))		cameraPosition += Vector3(0.0, 0.0, 1.0f);		//Q
	if(keyDown(0x45))		cameraPosition -= Vector3(0.0, 0.0, 1.0f);		//E
	//
	if(keyDown(0x57))		cameraPosition += cameraForward;								// W
	if(keyDown(0x41))		cameraPosition -= cameraForward*Vector3(0.0, 0.0, 1.0f);		// A
	if(keyDown(0x53))		cameraPosition -= cameraForward;								// S
	if(keyDown(0x44))		cameraPosition += cameraForward*Vector3(0.0, 0.0, 1.0f);		// D

	//

	float XAxisSensitivity = 0.003f;
	float YAxisSensitivity = 0.003f;
	
	POINT Cursor;
           
	GetCursorPos(&Cursor);
	SetCursorPos(128, 128);
   
	float DeltaX = (Cursor.x - 128) * XAxisSensitivity;
	float DeltaY = (Cursor.y - 128) * YAxisSensitivity;

	cameraTheta	+= DeltaX;
	cameraPhi	-= DeltaY;

	float Epsilon = 0.1f;

	if(cameraPhi >= +M_PI_2 - Epsilon) cameraPhi = +(float)M_PI_2 - Epsilon;
	if(cameraPhi <= -M_PI_2 + Epsilon) cameraPhi = -(float)M_PI_2 + Epsilon;
}

bool& Camera::UpdateVectors()
{
	return updateVectors;
}

void Camera::render(int x, int y)	// Supply xy data for pick.
{
	update();

	glMatrixMode(GL_PROJECTION);	// Select the projection matrix.
	glLoadIdentity();				// Reset the projection matrix.

	GLint viewport[4];
    glGetIntegerv (GL_VIEWPORT, viewport);

	GLint mode = 0;
	glGetIntegerv(GL_RENDER_MODE, &mode);
	
	if(mode == GL_SELECT)
	{
		// If we are picking generate a pick matrix.
		// We supply a bounding box of 2x2 pixels, to which rendering is constrained.

		gluPickMatrix((GLdouble)x, (GLdouble)y, 2.0, 2.0, viewport);
	}
	
	// Calculate the aspect ratio of the window.
	gluPerspective(45.0f,(float)viewport[2]/(float)viewport[3], 0.1f, 1000.0f);

	// Back to the model view...
	
	glMatrixMode(GL_MODELVIEW);		// Select the modelview matrix.
	glLoadIdentity();	
		
	// Reset the modelview matrix.

	if(UpdateVectors()) cameraForward = ComputeForward();	// Only re-calc forward vector based on angles if
															// asked to.

	LookAt(cameraPosition, cameraPosition+cameraForward, Vector3(0.0f, 0.0f, 1.0f));
}

Vector3 Camera::ComputeForward()
{
	return Vector3(sin(cameraTheta)*cos(cameraPhi), cos(cameraTheta)*cos(cameraPhi), sin(cameraPhi));
}

Vector3& Camera::Forward()
{
	return cameraForward;
}

Vector3& Camera::Position()
{
	return Camera::cameraPosition;
}

void Camera::setCapture(bool capture)
{
	if(captureInput != capture)
	{
		ShowCursor(!capture);

		POINT cursor;
           
		if(capture)
		{
			GetCursorPos(&cursor);

			oldCursorX = cursor.x;
			oldCursorY = cursor.y;
			
			SetCursorPos(128, 128);
		}
		else SetCursorPos(oldCursorX, oldCursorY);
	}
	
	captureInput = capture;
}

void Camera::capture()
{
	setCapture();
}

void Camera::release()
{
	setCapture(false);
}

bool Camera::captured()
{
	return captureInput;
}