#include "mesh.h"

Mesh::Mesh() : m_Texture(-1)
{
	setScale(1.0f);
	setSpecular(1.0f, 0.0f, 0.0f);
}

Mesh::Mesh(Vector3 v) : GLObject(v), m_Texture(-1)
{
	setScale(1.0f);
	setSpecular(1.0f, 0.0f, 0.0f);
}

void Mesh::Destroy()
{		
	vertices.clear();
	indices.clear();

	DestroyTexture();

	GLObject::Destroy();

	cout << "Mesh destroyed." << endl;
}

void Mesh::DestroyTexture()
{
	glDeleteTextures(1, &m_Texture);
}

bool Mesh::LoadTexture(string Path)
{
	if(!GLLoadBitmap(Path.c_str(), m_Texture))
	{
		cout << "File not found: " << Path << endl;
	}

	return true;
}

bool Mesh::LoadOFF(string path)	// OFF object loader.
{
	// First clear eveything in case we have data.

	Destroy();
	
	ifstream file(path.c_str());

	if(!file.good()) return false;

	string type;

	file >> type;

	if(type != "OFF") return false;	// Not as OFF file.

	int vcount = 0; // Vertices
	int fcount = 0; // Faces

	file >> vcount >> fcount;

	cout << "OFF" << endl << "Verts: " << vcount << endl << "Faces: " << fcount << endl;

	for(int i = 0; i < vcount; i++)
	{
		float x, y, z, u, v;
		
		file >> x >> y >> z >> u >> v;

		TLVertex vert;

		vert.m_X = x;
		vert.m_Y = y;
		vert.m_Z = z;

		vert.u = u;
		vert.v = v;

		vertices.push_back(vert);	// Add the vertex to our list.
	}

	// Now for faces...
	
	for(int i = 0; i < fcount; i++)
	{
		int n, a, b, c;

		// n is a dummy - we always have 3 verts per face.

		file >> n >> a >> b >> c;

		indices.push_back(a);
		indices.push_back(b);
		indices.push_back(c);
	}

	file.close(); // We are done with the file.

	// Calculate normals.

	for(int Index = 0; Index < (int)(indices.size()-3); Index += 3)
	{
		Vector3 A = vertices[indices[Index+0]];
		Vector3 B = vertices[indices[Index+1]];
		Vector3 C = vertices[indices[Index+2]];
		
		Vector3 Normal = ((B-A)*(C-A)).Normalize();

		vertices[indices[Index+0]].norm = Normal;
		vertices[indices[Index+1]].norm = Normal;
		vertices[indices[Index+2]].norm = Normal;
	}

	return true;
}

void Mesh::Render()
{
	if(!Visible()) return;

	GLint mode = 0;
	glGetIntegerv(GL_RENDER_MODE, &mode);
	
	if(mode == GL_SELECT) return;	// We are only selecting control points.
	
	save();

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);

	glEnable(GL_CULL_FACE);

	glBindTexture(GL_TEXTURE_2D, m_Texture);

	glBegin(GL_TRIANGLES);

	for(int Index = 0; Index < (int)indices.size(); Index++)
	{
		glTexCoord2f(vertices[indices[Index]].u, vertices[indices[Index]].v);
		glNormal3fv(vertices[indices[Index]].norm.m_V);
		glVertex3fv(vertices[indices[Index]].m_V);
	}

	glEnd();

	restore();
}