#include "terrain.h"
#define GL_CLAMP_TO_EDGE	0x812F	

Terrain::Terrain() : m_Size(0), m_IndexCount(0), m_pVertices(0), m_pNormals(0), m_pIndices(0)
{
	setScale(2, 2, 1);
}

Terrain::~Terrain()
{
	Destroy();
}

void Terrain::Destroy()
{
	delete [] m_pVertices;
	delete [] m_pNormals;
	delete [] m_pIndices;

	delete [] m_pTexCoordsX;
	delete [] m_pTexCoordsY;

	m_pVertices	= 0;
	m_pNormals	= 0;
	m_pIndices	= 0;

	m_pTexCoordsX = 0;
	m_pTexCoordsY = 0;

	DestroyTexture();

	cout << "Terrain destroyed." << endl;
}

void Terrain::DestroyTexture()
{
	glDeleteTextures(1, &m_Texture);
}

bool Terrain::LoadTexture(string Path)
{
	if(!GLLoadBitmap(Path.c_str(), m_Texture))
	{
		cout << "File not found: " << Path << endl;
	}

	cout << "Terrain textures loaded." << endl;

	return true;
}

bool Terrain::Load(string Path, string TexturePath)	// From Red Channel.
{
	ifstream File(Path.c_str());

	cout << "Reading file..." << endl;

	if(!File.good())
	{
		cout << "File read error." << endl;

		return false;
	}

	string Header;

	int Width		= 0;
	int Height		= 0;
	int Resolution	= 0;

	File >> Header >> Width >> Height >> Resolution;

	cout << "Header: " << Header << " Width: " << Width << " Height: " << Height << " Resolution: " << Resolution << endl;

	cout << "Loading data..." << endl;

	if(Width != Height)
	{
		cout << "Error: Width != Height" << endl;

		return false;
	}

	m_Size = Width;

	m_IndexCount = (m_Size-1)*(m_Size-1)*6;

	cout << "Index Calc: " << m_IndexCount << endl;
	
	m_pVertices = new Vector3[m_Size*m_Size];
	m_pNormals	= new Vector3[m_Size*m_Size];
	m_pIndices	= new int[m_IndexCount];

	m_pTexCoordsX	= new float[m_Size*m_Size];
	m_pTexCoordsY	= new float[m_Size*m_Size];

	// Vertices.

	for(int y = 0; y < m_Size; y++)
	{
		for(int x = 0; x < m_Size; x++)
		{
			int R, G, B;

			File >> R >> G >> B;

			float fX = (float)x - (float)m_Size/2.0f + 0.5f;
			float fY = (float)y - (float)m_Size/2.0f + 0.5f;
			float fZ = (float)(R - 128) / 5.0f;

			fX *= 2;
			fY *= 2;

			m_pVertices[y*Width+x] = Vector3(fX, fY, fZ);

			m_pTexCoordsX[y*Width+x] = (float)x / (float)m_Size;
			m_pTexCoordsY[y*Width+x] = -(((float)y / (float)m_Size) - 1.0f);

			//m_pTexCoordsX[y*Width+x] *= 5;	// Increase tiling.
		//	m_pTexCoordsY[y*Width+x] *= 5;
			m_pTexCoordsX[y*Width+x] *= 1;	// Do not tile...
			m_pTexCoordsY[y*Width+x] *= 1;
		}

		if(y % 100 == 0) cout << ((int)(y*100)/(int)(m_Size)) << "%" << endl;
	}

	// Indices.

	int Index = 0;

	for(int y = 0; y < m_Size-1; y++)
	{
		for(int x = 0; x < m_Size-1; x++)
		{
			m_pIndices[Index++] = (y+0)*m_Size+(x+0);
			m_pIndices[Index++] = (y+0)*m_Size+(x+1);
			m_pIndices[Index++] = (y+1)*m_Size+(x+0);

			m_pIndices[Index++] = (y+1)*m_Size+(x+0);
			m_pIndices[Index++] = (y+0)*m_Size+(x+1);
			m_pIndices[Index++] = (y+1)*m_Size+(x+1);
		}
	}

	cout << "Index Actual: " << Index << endl;

	// Smooth.

	for(int Index = 0; Index < m_IndexCount-3; Index++)
	{
		Vector3 A = m_pVertices[m_pIndices[Index+0]];
		Vector3 B = m_pVertices[m_pIndices[Index+1]];
		Vector3 C = m_pVertices[m_pIndices[Index+2]];
			
		float AvgZ = (0.5f)*A.m_Z + (0.25f)*B.m_Z + (0.25f)*C.m_Z;

		m_pVertices[m_pIndices[Index+0]].m_Z = AvgZ;
	}

	// Normals.

	for(int Index = 0; Index < m_IndexCount-3; Index++)
	{
		Vector3 A = m_pVertices[m_pIndices[Index+0]];
		Vector3 B = m_pVertices[m_pIndices[Index+1]];
		Vector3 C = m_pVertices[m_pIndices[Index+2]];
		
		Vector3 Normal = ((B-A)*(C-A)).Normalize();

		m_pNormals[m_pIndices[Index+0]] = Normal;
	}

	cout << "Loading texture/s..." << endl;

	LoadTexture(TexturePath);

	BuildDisplayList();

	cout << "Done." << endl;

	return true;
}

void Terrain::BuildDisplayList()
{
	if(!m_pVertices || !m_pNormals || !m_pIndices) return;

	glNewList(DISPLAY_LIST_ID, GL_COMPILE);	// Change the number later.
	
	save();

	transform();
	light();
	
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);

	glEnable(GL_CULL_FACE);

	glBindTexture(GL_TEXTURE_2D, m_Texture);

	glBegin(GL_TRIANGLES);

	for(int Index = 0; Index < m_IndexCount; Index++)
	{
		glTexCoord2f(m_pTexCoordsX[m_pIndices[Index]], m_pTexCoordsY[m_pIndices[Index]]);
		glNormal3fv(m_pNormals[m_pIndices[Index]].m_V);
		glVertex3fv(m_pVertices[m_pIndices[Index]].m_V);
	}

	glEnd();

	restore();

	glEndList();
}

void Terrain::Render()
{		
	GLint mode = 0;
	glGetIntegerv(GL_RENDER_MODE, &mode);
	
	if(mode == GL_SELECT) return;	// Nothing to select in the sky box
	glCallList(DISPLAY_LIST_ID);	// Change the number later.
}
