#include "vector3.h"

Vector3::Vector3()
{
	m_X = 0;
	m_Y = 0;
	m_Z = 0;
}

Vector3::Vector3(float X, float Y, float Z)
{
	m_X = X;
	m_Y = Y;
	m_Z = Z;
}

// Operators.

Vector3& Vector3::operator = (float Scalar)
{
	m_X = Scalar;
	m_Y = Scalar;
	m_Z = Scalar;

	return(*this);
}

Vector3 Vector3::operator + (Vector3& Vec)
{
	return Vector3(m_X + Vec.m_X, m_Y + Vec.m_Y, m_Z + Vec.m_Z);
}

Vector3& Vector3::operator += (Vector3& Vec)
{		
	m_X += Vec.m_X;
	m_Y += Vec.m_Y;
	m_Z += Vec.m_Z;

	return(*this);
}

Vector3 Vector3::operator - (Vector3& Vec)
{
	return Vector3(m_X - Vec.m_X, m_Y - Vec.m_Y, m_Z - Vec.m_Z);
}

Vector3& Vector3::operator -= (Vector3& Vec)
{
	m_X -= Vec.m_X;
	m_Y -= Vec.m_Y;
	m_Z -= Vec.m_Z;

	return(*this);
}

Vector3 Vector3::ElemMult (Vector3& Vec)
{
	return Vector3(m_X*Vec.m_X, m_Y*Vec.m_Y, m_Z*Vec.m_Z);
}

Vector3 Vector3::operator * (Vector3& Vec)
{
	return Vector3(m_Y*Vec.m_Z-m_Z*Vec.m_Y, m_Z*Vec.m_X-m_X*Vec.m_Z, m_X*Vec.m_Y-m_Y*Vec.m_X);
}

Vector3 Vector3::operator * (float Scalar)
{
	return Vector3(m_X * Scalar, m_Y * Scalar, m_Z * Scalar);
}

Vector3 Vector3::operator / (float Scalar)
{
	return Vector3(m_X / Scalar, m_Y / Scalar, m_Z / Scalar);
}

Vector3& Vector3::operator *= (float Scalar)
{
	m_X *= Scalar;
	m_Y *= Scalar;
	m_Z *= Scalar;

	return(*this);
}

Vector3& Vector3::operator /= (float Scalar)
{
	m_X /= Scalar;
	m_Y /= Scalar;
	m_Z /= Scalar;

	return(*this);
}

float Vector3::Length()
{
	return sqrtf((m_X*m_X + m_Y*m_Y + m_Z*m_Z));
}

float Vector3::LengthSquared()
{
	return (m_X*m_X + m_Y*m_Y + m_Z*m_Z);
}

Vector3& Vector3::Normalize()
{		
	return (*this *= (1.0f / Length()));
}

Vector3 Vector3::Normalized()
{
	return (*this * (1.0f / Length()));
}

float Vector3::Dot(Vector3& Vec)
{
	return m_X*Vec.m_X + m_Y*Vec.m_Y + m_Z*Vec.m_Z;
}

Vector3 Vector3::operator ^ (float Scalar)
{
	return Vector3(pow(m_X, Scalar), pow(m_Y, Scalar), pow(m_Z, Scalar));
}

bool Vector3::operator < (Vector3& Vec)
{
	return Length() < Vec.Length();
}

bool Vector3::operator > (Vector3& Vec)
{
	return Length() > Vec.Length();
}

//

Vector3 operator * (float Scalar, Vector3& Vec)
{
	return Vector3(Vec.m_X * Scalar, Vec.m_Y * Scalar, Vec.m_Z * Scalar);
}
