#include <vector>

using namespace std;

#include <math.h>

#include "vector3.h"
#include "globject.h"


class BezierSpline : public GLObject, public vector<GLObject*>
{
	vector<Vector3> C;		// Coeficients for parametic method.
	float m_dt;				// Step size for rendering.

	bool m_bCasteljou;		// Casteljau selected?
	bool m_bShowLines;

	vector<Vector3> casPoints;	// Tempory points for recusive bernstein (Casteljau)

public:
	
	// Constructor setting dt to a default value.
	BezierSpline(float m_dt = 0.01);

	~BezierSpline();

	void Destroy();


	void Time();
	// Get a reference to the step used in rendering.
	float& dt();

	// Update the coeficients for parametric method and ensure the temp. vector is the correct size.
	void build();

	// Reference to whether we should use Castejau.
	bool& Casteljau();

	// Reference to whether we should show control lines.
	bool& ShowLines();

	/*
	
	The actual bezier curve function. Given a value from 0 to 1 inclusive, it returns the bezier interpolated point.
	The result should be the same using parametric or Castejau.

	*/
	Vector3 f(float t);

	//Clear all the points.
	void clear();
	
	// Render the points and curve, calling f(t) to get plot the appropriate curve.
	virtual void Render();

private:

	// Factorial function used by parametric method.
	int fact(int j);

	// Coeficient calculation.
	Vector3 Cj(int j);

	// The polynomial which generates a point on the bezier curve, based on the value of t.
	Vector3 fPoly(float t);

	// Casteljau method. [j] is the number of points to process, held in the vector, [casPoints].
	Vector3 fCasteljau(float t, int j);	// Recursive bernstein

	
		float startCJ, endCJ, startPara, endPara, CJtime, ParaTime;
};