#pragma once

#include "base.h"

#include "vector3.h"
#include "globject.h"
#include "bezierspline.h"
#include "camera.h"
#include "world.h"
#include "terrain.h"
#include "mesh.h"
#include "skybox.h"
#include <time.h>

class Sphere : public GLObject
{
public:

	Sphere();
	
	Sphere(Vector3 v);

	virtual void Render();
};

class Light : public Sphere
{
	GLfloat ambient[4];
	float linear;		// Linear light fall off.

	int lightId;

public:

	Light();
	Light(Vector3 v);
	Light(int id);

	int& id();

	float& Linear();

	void setAmbient(float r, float g, float b, float a=0.5f);

	virtual void Render();
};

class ControlPoint : public Sphere
{
	bool selected;

public:

	ControlPoint();
	ControlPoint(Vector3 v);
	
	virtual void Render();
	void select();
	void deSelect();
};

class Fighter : public Mesh
{
	Vector3 At;	// Fighter's front vector
	Vector3	Up;	// Fighter roll.

	Camera* trackingCamera;

public:

	Fighter();
	Fighter(Vector3 v);

	virtual void Destroy();

	void Create();

	virtual void Render();

	Vector3& at();

	void Track(Camera* camera);

	void AlignToCamera();
};

class World : public vector<GLObject*>
{
protected:
	
	BezierSpline	spline;
	Terrain			terrain;
	SkyBox			skybox;

	Fighter			fighter;		// Object to move along bezier.

	Light			light1;
	Light			light2;
	Light			light3;

	float w;	// Current step along curve [0, 1]
	float dt;	// Time step
	float u;	// Current velocity
	float g;	// Gravity

	bool paused;
	bool physicsEnabled;
	bool looped;

public:


	World();

	void Destroy();

	void Create(Camera* pCamera);
	
	void Render();

	GLObject* PickObject(Camera& camera, int x, int y);	// Use the camera to pick
														// out an object based on a viewport point.

	bool IsControlPoint(GLObject* obj);

	bool IsLight(GLObject* obj);

	void Step();

	void AlignFighter();	// Align the nose of the fighter correctly.

	bool& Paused();

	bool& PhysicsEnabled();

	bool& Looped();
};